package org.netkernel.layer0.bnf;

import java.io.IOException;
import java.io.StringWriter;

import org.netkernel.container.IMessages;
import org.netkernel.layer0.meta.GrammarException;
import org.netkernel.layer0.util.OrderedCheapMultiStringMap;


public class Interleave extends GroupImpl
{	
	private int[] mPartGroupIndices;
	
	public Interleave(IGrammarGroup aParent)
	{	super(aParent,null,null);
	}
	
	public boolean needsToMatchDeeper()
	{	return true;
	}
	
	public void writeXML(StringWriter aSW) throws IOException
	{	aSW.write("<interleave>");
		for (int i=0; i<getPartCount(); i++)
		{	IGrammarPart p = getPart(i);
			if (p instanceof IGrammarGroup)
			{	p.writeXML(aSW);
			}
			else
			{	aSW.write("<group>");
				p.writeXML(aSW);
				aSW.write("</group>");
			}
		}
		aSW.write("</interleave>");
	}

	protected void buildRegex(StringWriter aSW)
	{
		NamedGroups innerGroups=new NamedGroups();

		StringWriter sw = new StringWriter(128);
				
		int count=getPartCount();
		mPartGroupIndices = new int[count];
		
		sw.write('(');
		for (int i=0; i<count; i++)
		{	sw.write('(');
			innerGroups.skipGroup();
			mPartGroupIndices[i]=innerGroups.size()-1;
			getPart(i).writeRegex(sw, innerGroups );
			sw.write(')');
			if (i<count-1)
			{	sw.write('|');
			}
		}
		sw.write(')');
		String innerRegexString=sw.toString();
		setInnerRegex(innerRegexString);
		
		aSW.write('(');
		aSW.write(innerRegexString);
		aSW.write("*)");
		
		mGroups.putGroup(this);
		mGroups.skipGroups(innerGroups.size()+1);
	}
	
	public boolean matches(String[] aCapturedGroups, int aOffset)
	{	
		boolean result=true;
		String cp = aCapturedGroups[aOffset];
		if (cp==null)
		{	//nothing matched at all
			return false;
		}
		String[][] matches = findMatchingGroups(cp);
		int[] partCounts = new int[getPartCount()];
		for (int j=0; j<matches.length && result; j++)
		{	String[] cgs = matches[j];
			for (int i=0; i<mPartGroupIndices.length; i++)
			{	int partIndex=mPartGroupIndices[i];
				String c=cgs[partIndex+1];
				if (c!=null)
				{	partCounts[i]++;
					IGrammarPart part = getPart(i);
					if (part instanceof IGrammarGroup)
					{	IGrammarGroup g = (IGrammarGroup)part;
						if (g.needsToMatchDeeper())
						{	result = g.matches(cgs, partIndex+2);
						}
					}
					break;
				}
			}
		}
		
		// compare part count against expected
		if (result)
		{
			for (int i=0; i<partCounts.length; i++)
			{	int count = partCounts[i];
				IGrammarPart part = getPart(i);
				if (part instanceof RegularGroup)
				{	RegularGroup g = (RegularGroup)part;
					if (count>g.getMax() || count<g.getMin())
					{	result=false;
					}
				}
				else
				{	if (count!=1)
					{	result=false;
					}
				}
			}
		}
		
	
		return result;
	}	
	
	public void parse(String[] aCapturedGroups, OrderedCheapMultiStringMap aResults, int aOffset)
	{	
		String cp=aCapturedGroups[aOffset];
		if (cp==null)
		{	return;
		}
		String[][] matches = findMatchingGroups(cp);
		for (int j=0; j<matches.length; j++)
		{	String[] cgs = matches[j];
			for (int i=0; i<mPartGroupIndices.length; i++)
			{	int partIndex=mPartGroupIndices[i];
				String c=cgs[partIndex+1];
				if (c!=null)
				{	IGrammarPart part = getPart(i);
					if (part instanceof IGrammarGroup)
					{	((IGrammarGroup)part).parse(cgs, aResults, partIndex+1);
					}
					break;
				}
			}
		}
	}

	
	public String constructIdentifier(OrderedCheapMultiStringMap aPartsRemaining, IMessages aMessages) throws GrammarException
	{
		int count=getPartCount();
		StringWriter sw = new StringWriter(128);
		for (int i=0; i<count; i++)
		{	Object part = getPart(i);
			if (part instanceof StaticPart)
			{	sw.write( ((StaticPart)part).getContent() );
			}
			else if (part instanceof RegexPart)
			{	throw new GrammarException(aMessages.format("MSG_BNF_AMBIGUOUS", ((RegexPart)part).getRegex()));
			}
			else if (part instanceof IGrammarGroup)
			{	String subGroup=((IGrammarGroup)part).constructIdentifier(aPartsRemaining,aMessages);
				sw.write(subGroup);
			}
				
		}
		return sw.toString();
	}

}