package org.netkernel.layer0.representation;

import java.util.Comparator;


/** IHDSNodeList is an ordered list of IHDSNodes
 * @author tab
 */
public interface IHDSNodeList extends Iterable<IHDSNode>
{
	/** Return the size of the list
	 */
	int size();
	
	/** Return an item from the list
	 * @param aIndex the index of the item
	 */
	IHDSNode get(int aIndex);
	
	/** Filter the nodes in the list
	 * @param aPredicate the predicate used to determine inclusion
	 * @return the filtered list
	 */
	IHDSNodeList filter(IHDSPredicate aPredicate);
	
	/** Sort the nodes in the list
	 * @param aComparator the comparator used to determine order of the items
	 * @return the sorted list
	 */
	IHDSNodeList sort(Comparator<IHDSNode> aComparator);
	
	/** Return a list of all the nodes from all nodes in the list which match the given XPath
	 * @param aXPath the HDS XPath expression
	 * @return the list of nodes
	 */
	IHDSNodeList getNodes(String aXPath);
	
	/** Return the first node derived from any of the nodes in the list which matches the given XPath 
	 * @param aXPath the HDS XPath expression
	 * @return the first matching node or null
	 */
	IHDSNode getFirstNode(String aXPath);
	
	/** Return the first node in the list
	 */
	IHDSNode getFirstNode();
	
	/** Return the node value of the first node derived from any of the nodes in the list which matches the given XPath 
	 * @param aXPath aXPath the HDS XPath expression
	 */
	Object getFirstValue(String aXPath);
	
	/** Return the value of the first node in the list
	 * @return
	 */
	Object getFirstValue();
}
